import logging
import typing as t
import numpy as np
import aiohttp
from .ws import WS
from .events import Event
from .utils import get_random_username
from typing import Union, List, Optional
from .rest import RestApi

_LOG = logging.getLogger("kxspy.client")

class Client:
    """
    The main class.
    """
    def __init__(
        self,
        ws_url: str = "wss://network.kxs.rip/",
        rest_url: str = "https://network.kxs.rip",
        username: str = get_random_username(),
        enablevoicechat: bool = False,
        exchangekey: str = None,
        isMobile: bool = False,
        isSecure: bool = True,
        admin_key: str = None,
        connect: bool = True,
        session: t.Optional[aiohttp.ClientSession] = None
    ) -> None:
        self.ws = WS(
            ws_url=ws_url,
            username=username,
            enable_voice_chat=enablevoicechat,
            exchange_key=exchangekey,
            connect=connect,
            isMobile=isMobile,
            isSecure=isSecure,
            session=session
        )
        self.username = username
        self.rest = RestApi(rest_url,admin_key,session)
        self.emitter = self.ws.emitter
        self._registered_listeners: t.List[t.Tuple[t.Any, t.Callable, t.Type[Event]]] = []


    def add_event_hooks(self, obj):
        """
        Scans the provided class ``obj`` for functions decorated with :func:`listener`,
        and sets them up to process Lavalink events.
        """
        for attr_name in dir(obj):
            method = getattr(obj, attr_name)
            events = getattr(method, "_kxspy_events", None)
            if not events:
                continue

            if not events:
                self.emitter.add_listener(event=None, func=method)
            else:
                for ev in events:
                    self.emitter.add_listener(event=ev, func=method)

    def remove_event_hooks(self, obj: t.Any):
        """
        Removes all previously registered listeners for an object.
        """
        to_remove = [r for r in self._registered_listeners if r[0] == obj]
        for _, method, ev in to_remove:
            try:
                self.emitter.remove_listener(event=ev, func=method)
                _LOG.debug(f"Removed listener: {method.__name__} for {ev}")
            except Exception as e:
                _LOG.warning(f"Failed to remove listener {method.__name__}: {e}")

            self._registered_listeners.remove((_, method, ev))

    async def connect(self):
        """Connect to Kxs Network."""
        await self.ws.connect()

    async def close(self):
        """Close connection to Kxs Network."""
        await self.ws.close()

    async def join_game(self, gameId):
        """Join a game by its ID."""
        await self.ws.send({"op": 3, "d": {"gameId": gameId,"user": self.username}})

    async def leave_game(self):
        """Leave the current game."""
        await self.ws.send({"op": 4, "d": {}})

    async def report_kill(self,killer: str, killed: str):
        """Report a kill in the game"""
        await self.ws.send({"op": 5, "d": {"killer":killer,"killed":killed}})

    async def check_version(self):
        """Check for the latest version of Kxs"""
        await self.ws.send({"op": 6, "d": {}})

    async def send_message(self,text: str):
        """Send a message to the in-game chat"""
        await self.ws.send({"op": 7, "d": {"text":text}})

    async def update_voicechat(self,isVoiceChat: bool):
        """Update the voice chat status"""
        await self.ws.send({"op": 98, "d": {"isVoiceChat":isVoiceChat}})

    async def send_voicedata(self, audio_data: Union[bytes, bytearray,np.ndarray, List[int]], user_id: Optional[str] = None):
        """Update the voice chat status"""
        if isinstance(audio_data, (bytes, bytearray)):
            int16_array = np.frombuffer(audio_data, dtype=np.int16)
            data_to_send = int16_array.tolist()
        elif isinstance(audio_data, np.ndarray) and audio_data.dtype == np.int16:
            data_to_send = audio_data.tolist()
        elif isinstance(audio_data, list):
            data_to_send = audio_data
        else:
            raise TypeError(
                "audio_data must be bytes, bytearray, numpy.ndarray[int16], or list[int]"
            )
        await self.ws.send({"op": 99, "d": data_to_send, "u":user_id or self.ws.uuid})

    async def ws_latency(self):
        """Send the latency of websocket"""
        return await self.ws.measure_latency()




import asyncio
import logging
import aiohttp
import kxspy
from .emitter import Emitter
from time import perf_counter
from .utils import get_random_username
from .events import *


_LOG = logging.getLogger("kxspy.ws")

MESSAGE_QUEUE_MAX_SIZE = 25

OP_EVENT_NAMES = {
    1: "HeartBeatEvent",
    2: "IdentifyEvent",
    3: "GameStart",
    4: "GameEnd",
    5: "KillEvent",
    6: "VersionUpdate",
    7: "ChatMessage",
    10: "HelloEvent",
    12: "ExchangejoinEvent",
    13: "ExchangeOnlineEvent",
    14: "ExchangeOfflineEvent",
    15: "ExchangeGameAliveEvent",
    16: "ExchangeGameEnd",
    87: "BroadCasteEvent",
    98: "VoiceChatUpdate",
    99: "VoiceData",
}

class WS:
    """Handles the WebSocket connection to the Kxs network."""
    def __init__(
        self,
        ws_url: str = "wss://network.kxs.rip/",
        username: str | None = None,
        enable_voice_chat: bool = False,
        exchange_key: str | None = None,
        connect: bool = True,
        isMobile: bool = False,
        isSecure: bool = True,
        session: aiohttp.ClientSession | None = None
    ):
        self.ws_url = ws_url
        self.username = username or get_random_username()
        self.enable_voice_chat = enable_voice_chat
        self.exchange_key = exchange_key
        self.isMobile = isMobile
        self.isSecure = isSecure

        self._loop = asyncio.get_event_loop()
        self._session = session or aiohttp.ClientSession()
        self._ws: aiohttp.ClientWebSocketResponse | None = None
        self._message_queue: list[dict] = []
        self._listen_task: asyncio.Task | None = None
        self._heartbeat_task: asyncio.Task | None = None
        self._destroyed = False

        self.is_connect: bool = False
        self.is_authenticated: bool = False
        self._uuid = None

        self.emitter = Emitter()
        self.version = f"kxspy/{kxspy.__version__}"

        if connect:
            self.connect()

    def connect(self) -> asyncio.Task:
        if self._destroyed:
            raise IOError("Cannot connect: transport destroyed.")

        if self._listen_task and not self._listen_task.done():
            self._listen_task.cancel()

        return self._loop.create_task(self._connect())

    async def _connect(self):
        await self.close()

        attempt = 0
        while not self._destroyed and not self.is_connect:
            attempt += 1
            try:
                _LOG.info(f"Connecting to WebSocket: {self.ws_url}")
                self._ws = await self._session.ws_connect(self.ws_url, heartbeat=60)
                self.is_connect = True
                _LOG.info("WebSocket connection established.")
                self._listen_task = self._loop.create_task(self._listen())
                break
            except (aiohttp.ClientError, asyncio.TimeoutError) as exc:
                delay = min(10 * attempt, 60)
                _LOG.error(
                    f"Connection failed ({type(exc).__name__}: {exc}), retrying in {delay}s..."
                )
                await asyncio.sleep(delay)
            except (aiohttp.ClientConnectorError, aiohttp.WSServerHandshakeError,
                    aiohttp.ServerDisconnectedError) as error:
                if isinstance(error, (aiohttp.ClientConnectorError, aiohttp.ServerDisconnectedError)):
                    delay = min(10 * attempt, 60)
                    _LOG.error(
                        f"Connection failed ({type(error).__name__}: {error}), retrying in {delay}s..."
                    )
                    await asyncio.sleep(delay)
                else:
                    _LOG.error(f"Exception WS: {error}")

    async def measure_latency(self, timeout: float = 5.0) -> float:
        if not self.is_connected:
            raise ConnectionError("WebSocket is not connected.")

        fut = asyncio.get_event_loop().create_future()
        start = perf_counter()

        async def on_version_update(_event):
            try:
                self.emitter.remove_listener("VersionUpdate", on_version_update)
            except ValueError:
                pass
            if not fut.done():
                fut.set_result((perf_counter() - start) * 1000)

        self.emitter.add_listener("VersionUpdate", on_version_update)
        await self.send({"op": 6, "d": {}})

        try:
            latency = await asyncio.wait_for(fut, timeout)
            return latency
        except asyncio.TimeoutError:
            _LOG.error("Timeout : VersionUpdate did not arrive in time ( mesure_latency ).")
            return None

    async def close(self, code=aiohttp.WSCloseCode.OK):
        if self._listen_task:
            self._listen_task.cancel()
        if self._heartbeat_task:
            self._heartbeat_task.cancel()

        if self._ws:
            try:
                await self._ws.close(code=code)
            except Exception:
                pass
            finally:
                self._ws = None
                self.is_connect = False

    async def destroy(self):
        self._closing = True

        tasks = []
        for task in [self._listen_task, self._heartbeat_task]:
            if task and not task.done():
                task.cancel()
                tasks.append(task)

        if self._ws and not self._ws.closed:
            await self._ws.close()

        if self._session and not self._session.closed:
            await self._session.close()

        if tasks:
            await asyncio.gather(*tasks, return_exceptions=True)

        self._ws = None
        self.is_connect = False
        self._destroyed = True
        _LOG.info("Kxspy WS destroyed cleanly.")


    async def _listen(self):
        assert self._ws is not None
        try:
            async for msg in self._ws:
                if msg.type == aiohttp.WSMsgType.TEXT:
                    _LOG.debug(f"Received message: {msg.data}")
                    self._loop.create_task(self._handle_message_safe(msg))
                elif msg.type in (
                    aiohttp.WSMsgType.CLOSE,
                    aiohttp.WSMsgType.CLOSING,
                    aiohttp.WSMsgType.CLOSED,
                ):
                    _LOG.warning("WebSocket closed by server.")
                    break
                elif msg.type == aiohttp.WSMsgType.ERROR:
                    _LOG.error(f"WebSocket error: {msg.data}")
                    break
        except asyncio.CancelledError:
            pass
        except Exception as e:
            _LOG.exception(f"Unexpected error while listening websocket message: {e}")

        self.is_connect = False
        self._ws = None
        if not self._destroyed:
            _LOG.info("Reconnecting after disconnection...")
            self._loop.create_task(self._connect())

    async def _handle_message_safe(self, msg: aiohttp.WSMessage):
        try:
            await self._handle_message(msg.json())
        except Exception:
            _LOG.exception("Error while handling websocket message")

    async def _handle_message(self, payload: dict):
        op = payload.get("op")
        d = payload.get("d", {})

        if d.get("error", None) is not None:
            event_name = OP_EVENT_NAMES.get(op, f"UnknownEvent(op={op})")
            self.emitter.emit(
                "ErrorEvent",
                ErrorEvent(event=event_name, error=d.get("error", "Unknown error"),op=op)
            )
            return
        if op == 1:  # Heartbeat
            self.emitter.emit("HeartBeatEvent", HeartBeatEvent.from_kwargs(**d))
        elif op == 2:  # Identify
            self._uuid = d.get("uuid")
            self.emitter.emit("IdentifyEvent", IdentifyEvent.from_kwargs(**d))
        elif op == 3:  # Game start
            if d.get("system", None) is not None:
                self.emitter.emit("GameStart", GameStart.from_kwargs(**payload["d"]))
            else:
                self.emitter.emit("ConfirmGameStart", ConfirmGameStart.from_kwargs(**payload["d"]))
        elif op == 4:  # Game end
            if d.get("left", None) is not None:
                self.emitter.emit("GameEnd", GameEnd.from_kwargs(**payload["d"]))
            else:
                self.emitter.emit("ConfirmGameEnd", ConfirmGameEnd.from_kwargs(**payload["d"]))
        elif op == 5: # KILL EVENT
            self.emitter.emit("KillEvent", KillEvent.from_kwargs(**d))
        elif op == 6: # VERSION UPDATE
            self.emitter.emit("VersionUpdate", VersionUpdate.from_kwargs(**d))
        elif op == 7: # CHAT MESSAGE
            if d.get("user", None) is not None:
                self.emitter.emit("ChatMessage", ChatMessage.from_kwargs(**payload["d"]))
            else:
                self.emitter.emit("ConfirmChatMessage", ConfirmChatMessage.from_kwargs(**payload["d"]))
        elif op == 10:  # Hello (heartbeat interval)
            interval = d.get("heartbeat_interval", 3000)
            await self.send({"op": 2,"d":{"username":self.username,"isVoiceChat":self.enable_voice_chat,"v":self.version,"isMobile":self.isMobile,"isSecure":self.isSecure,"exchangeKey":self.exchange_key}})
            await self._start_heartbeat(interval)
            self.emitter.emit("HelloEvent", HelloEvent.from_kwargs(**d))
        elif op == 12: # EXCHANGE KEY JOIN
            self.emitter.emit("ExchangejoinEvent", ExchangejoinEvent.from_kwargs(**d))
        elif op == 13: # EXCHANGE KEY ONLINE
            self.emitter.emit("ExchangeOnlineEvent", ExchangeOnlineEvent.from_kwargs(**d))
        elif op == 14: # EXCHANGE KEY OFFLINE
            self.emitter.emit("ExchangeOfflineEvent", ExchangeOfflineEvent.from_kwargs(**d))
        elif op == 15: # GAME ALIVE EXCHANGE KEY
            self.emitter.emit("ExchangeGameAliveEvent", ExchangeGameAliveEvent.from_kwargs(**d))
        elif op == 16: # GAME END EXCHANGE KEY
            d["data"]["stuff"] = Stuff.from_kwargs(**d["data"]["stuff"])
            self.emitter.emit("ExchangeGameEnd", ExchangeGameEnd.from_kwargs(**d["data"]))
        elif op == 87: # BROADCAST MESSAGE
            self.emitter.emit("BroadCasteEvent", BroadCasteEvent.from_kwargs(**d))
            _LOG.info("Received BroadcastEvent (op 87).")
        elif op == 98: # VOICE CHAT UPDATE
            if d.get("user", None) is not None:
                self.emitter.emit("VoiceChatUpdate", VoiceChatUpdate.from_kwargs(**payload["d"]))
            else:
                self.emitter.emit("ConfirmVoiceChatUpdate", ConfirmVoiceChatUpdate.from_kwargs(**payload["d"]))
        elif op == 99: # VOICE DATA
            self.emitter.emit("VoiceData", VoiceData(d=d, u=payload.get("u")))
        else:
            _LOG.warning(f"Unknown opcode: {op} — payload: {payload}")

    async def send(self, payload: dict):
        if not self.is_connect or not self._ws:
            if len(self._message_queue) >= MESSAGE_QUEUE_MAX_SIZE:
                _LOG.warning("Message queue full, discarding payload.")
                return
            _LOG.debug("Queueing payload until reconnected.")
            self._message_queue.append(payload)
            return

        try:
            await self._ws.send_json(payload)
        except ConnectionResetError:
            _LOG.warning("Connection reset during send, requeueing payload.")
            self._message_queue.append(payload)
            await self._connect()


    async def _start_heartbeat(self, interval: int):
        if self._heartbeat_task and not self._heartbeat_task.done():
            self._heartbeat_task.cancel()
        self._heartbeat_task = self._loop.create_task(self._heartbeat(interval))

    async def _heartbeat(self, interval: int):
        while self.is_connect and self._ws and not self._ws.closed:
            try:
                await asyncio.sleep(interval / 3000)
                await self.send({"op": 1, "d": {}})
            except asyncio.CancelledError:
                break
            except Exception as e:
                _LOG.error(f"Heartbeat error: {e}")
                break

    @property
    def is_connected(self) -> bool:
        return self.is_connect and self._ws.closed is False

    @property
    def uuid(self) -> bool:
        return self._uuid


"""
kxspy
---------
:copyright: (c) 2025-2026 lavecat
:license: MIT, see LICENSE for more details.
"""

__title__ = 'kxspy'
__author__ = 'lavecat'
__license__ = 'MIT'
__copyright__ = 'Copyright 2025-present lavecat'
__version__ = '1.0.5b'

from .client import Client
from .objects import *
from .events import *
from .rest import RestApi
from typing import Type, Callable



# https://github.com/devoxin/Lavalink.py/blob/development/lavalink/__init__.py#L28-L60
def listener(*events: Type[Event]):
    """
    Marks this function as an event listener for Kxspy.

    Example:
        @listener()
        async def on_any_event(self, event): ...

        @listener(ExchangeGameEnd)
        async def on_game_end(self, event: ExchangeGameEnd): ...
    """
    def wrapper(func: Callable):
        setattr(func, "_kxspy_events", events)
        return func
    return wrapper


import aiohttp
import logging
from time import time

_LOG = logging.getLogger("kxspy.rest")

class RestApi:
    """
    The class for the REST API of kxspy.

    Parameters
    ---------
    kxs_network_rest_url: :class:`str`
        Rest url of the kxs network REST API.
    adminKey: :class:`str`
        Only for admin routs .
    """
    def __init__(self, kxs_network_rest_url: str = "https://network.kxs.rip", adminKey: str = None, session: aiohttp.ClientSession = None) -> None:
        self.rest_uri = kxs_network_rest_url
        self.admin_key = adminKey
        self.session = session or aiohttp.ClientSession()

    async def request(self, method: str, rout: str, data: dict = {}) -> dict or str:
        """
        This function makes a request to the kxs network REST API.

        Parameters
        ---------
        method: :class:`str`
            method for request like `GET` or `POST` etc.
        rout: :class:`str`
            The route for request.
        data: :class:`dict`
            The data for request.

        Returns
        -------
        :class:`dict` or :class:`str`
            The response from the request.
        """
        rout = rout
        async with self.session.request(method, self.rest_uri + rout,json=data) as _response:
            _LOG.debug(f"{method} {self.rest_uri + rout}")
            if _response.content_type == "text/plain":
                return await _response.text()

            response = await _response.json()

            _LOG.debug(response)

            if _response.status != 200:
                _LOG.error(f"Request failed: {response}")
            return response


    async def online_count(self) -> dict:
        """
        This function makes a request to the kxs network REST API.

        Returns
        -------
        :class:`dict`
            The response from the request.
        """
        res = await self.request("GET", "/online-count")
        return res

    async def ig_count(self, gameId: str) -> dict:
        """
        This function makes a request to the kxs network REST API.

        Parameters
        ---------
        gameId: :class:`str`
            The game id.

        Returns
        -------
        :class:`dict`
            The response from the request.
        """
        res = await self.request("GET", f"/ig-count/{gameId}")
        return res

    async def getLatestVersion(self) -> str:
        """
        This function makes a request to the kxs network REST API.

        Returns
        -------
        :class:`str`
            The response from the request.
        """
        res = await self.request("GET", "/getLatestVersion")
        return res

    async def on_join(self, gameid: int, exchangeKey: str) -> dict:
        """
        This function makes a request to the kxs network REST API.

        Returns
        -------
        :class:`str`
            The response from the request.
        """
        res = await self.request("GET", f"/exchange/joined/{gameid}/{exchangeKey}")
        return res

    async def user_manager_status(self) -> dict:
        """
        This function makes a request to the kxs network REST API ( kxs admin endpoint ).

        Returns
        -------
        :class:`dict`
            The response from the request.
        """
        res = await self.request("POST", "/users-manager/status",data={"adminKey":self.admin_key})
        return res

    async def broadcast(self, msg: str) -> dict:
        """
        This function makes a request to the kxs network REST API ( kxs admin endpoint ).

        Returns
        -------
        :class:`dict`
            The response from the request.
        """
        res = await self.request("POST", "/broadcast",data={"adminKey":self.admin_key,"msg":msg})
        return res

    async def blacklist(self, ip: str,reason: str) -> dict:
        """
        This function makes a request to the kxs network REST API ( kxs admin endpoint ).

        Returns
        -------
        :class:`dict`
            The response from the request.
        """
        res = await self.request("POST", "/users-manager/blacklist",data={"adminKey":self.admin_key,"ip":ip,"reason":reason})
        return res

    async def unblacklist(self, ip: str) -> dict:
        """
        This function makes a request to the kxs network REST API ( kxs admin endpoint ).

        Returns
        -------
        :class:`dict`
            The response from the request.
        """
        res = await self.request("POST", "/users-manager/unblacklist",data={"adminKey":self.admin_key,"ip":ip})
        return res


    async def get_rest_latency(self) -> float:
        """|coro|

        Measures the REST latency for this node.
        This simply calls :func:`get_version` but measures the time between when the request was made,
        and when a response was received.

        Returns
        -------
        float
            The latency, in milliseconds. ``-1`` if an error occurred during the request (e.g. node is unreachable),
            otherwise, a positive number.
        """
        start = time()

        await self.online_count()

        return (time() - start) * 1000


from inspect import signature
from dataclasses import dataclass

# https://stackoverflow.com/questions/55099243/python3-dataclass-with-kwargsasterisk
class BaseObject:
    @classmethod
    def from_kwargs(cls, **kwargs):
        # fetch the constructor's signature
        cls_fields = {field for field in signature(cls).parameters}

        # split the kwargs into native ones and new ones
        native_args, new_args = {}, {}
        for name, val in kwargs.items():
            if name in cls_fields:
                native_args[name] = val
            else:
                new_args[name] = val

        # use the native ones to create the class ...
        ret = cls(**native_args)

        # ... and add the new ones by hand
        for new_name, new_val in new_args.items():
            setattr(ret, new_name, new_val)
        return ret

@dataclass
class Stuff(BaseObject):
    main_weapon: str
    secondary_weapon: str
    soda: int
    melees: str
    grenades: str
    medkit: int
    bandage: int
    pills: int
    backpack: str
    chest: str
    helmet: str


from .objects import BaseObject , Stuff
from dataclasses import dataclass , field
from typing import Any

class Event(BaseObject):
    """
    The class is a base event for websocket.
    """

@dataclass
class IdentifyEvent(Event):
    """
    IdentifyEvent. call when the websocket is ready.
    """
    uuid: str

@dataclass
class ExchangejoinEvent(Event):
    """
    Event on exchange join.
    """
    gameId: str
    exchangeKey: str

@dataclass
class ExchangeOnlineEvent(Event):
    """
    Event on exchange key online.
    """
    username: str
    v: str

@dataclass
class ExchangeOfflineEvent(Event):
    """
    Event on exchange key offline.
    """
    username: str

@dataclass
class ExchangeGameAliveEvent(Event):
    """
    Event on exchange key Game Alive.
    """
    alive: int

@dataclass
class ExchangeGameEnd(Event):
    """
    Event on exchange key Game End.
    """
    username: str
    kills: int
    damageDealt: int
    damageTaken: int
    duration: str
    position: str
    isWin: bool
    stuff: Stuff

@dataclass
class BroadCasteEvent(Event):
    """
    Event on broadcaste.
    """
    msg: str

@dataclass
class HelloEvent(Event):
    """
    Event on hello.
    """
    heartbeat_interval: int

@dataclass
class HeartBeatEvent(Event):
    """
    Event on heartbeat.
    """
    ok: bool
    count: int
    players: list

@dataclass
class ConfirmGameStart(Event):
    """
    Event on ConfirmGameStart.
    """
    ok: bool
    usernameChanged: bool

@dataclass
class GameStart(Event):
    """
    Event on GameStart.
    """
    ok: bool
    system: bool
    players: list

@dataclass
class GameEnd(Event):
    """
    Event on GameEnd.
    """
    left: str

@dataclass
class ConfirmGameEnd(Event):
    """
    Event on GameEnd.
    """
    ok: bool

@dataclass
class KillEvent(Event):
    """
    Event on KillEvent.
    """
    killer: str
    killed: str
    timestamp: int

@dataclass
class VersionUpdate(Event):
    """
    Event on VersionUpdate.
    """
    v: str

@dataclass
class ChatMessage(Event):
    """
    Event on ChatMessage.
    """
    user: str
    text: str
    timestamp: int
    system: bool

@dataclass
class ConfirmChatMessage(Event):
    """
    Event on ChatMessageConfirm.
    """
    ok: bool

@dataclass
class VoiceData(Event):
    """
    Event on VoiceData.
    """
    d: list
    u: str

@dataclass
class VoiceChatUpdate(Event):
    """
    Event on VoiceChatUpdate.
    """
    user: str
    isVoiceChat: bool

@dataclass
class ConfirmVoiceChatUpdate(Event):
    """
    Event on ConfirmVoiceChatUpdate.
    """
    ok: bool

@dataclass
class ErrorEvent(Event):
    """
    Event on ErrorEvent.
    """
    op: int
    event: str
    error: str


from setuptools import setup
import pathlib
import re

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')

version = ''

with open('kxspy/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Version is not set')

setup(
    name='kxspy',
    version=version,
    description='Kxspy is a async python client for KxsClient Network.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/lavecat/Kxspy',
    author='lavecat',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        'Programming Language :: Python :: 3 :: Only',
    ],
    keywords='kxsclient, surviv, kxspy, kxs, kxsnetwork',
    packages=["kxspy"],
    install_requires=["aiohttp","numpy"],
    project_urls={
        'Bug Reports': 'https://github.com/lavecat/Kxspy/issues',
        'Source': 'https://github.com/lavecat/Kxspy',
        'Documentation': 'https://kxspy.readthedocs.io/'
    },
)
